==> queueshare-0.0.55.tgz: package/src/QsYtAudioUriOrNull.js <==
import bufferHttpResBody from './bufferHttpResBody.js'
import HttpResAsJsonValue from './HttpResAsJsonValue.js'
import HttpsGetRes from './HttpsGetRes.js'
import qsWriteDebuggingInfo from './qsWriteDebuggingInfo.js'
import QueryString from './QueryString.js'
import SubstringByCircumfix from './SubstringByCircumfix.js'
import ThroweeAsJsonValue from './ThroweeAsJsonValue.js'
import Ungzipped from './Ungzipped.js'

let playerJsCacheOrNull = null
// todo use a uri->info map instead of caching just 1 uri? need to look at real-world debugging info to decide
const QsYtAudioUriOrNull = async (video) => {
	let watchResOrNull = null
	let watchResBodyOrNull = null
	let playerJsResOrNull = null
	let playerJsResBodyOrNull = null
	let uriOrNull = null
	let threw = false
	let throweeIfThrew
	try {
		watchResOrNull = await HttpsGetRes(
			'https://www.youtube.com/watch?' + QueryString({v: video}),
			{headers: {'Accept-Encoding': 'gzip'}},
		)
		watchResBodyOrNull = await bufferHttpResBody(watchResOrNull)
		const watchHtml = (await Ungzipped(watchResBodyOrNull)).toString()
		const playerJsUri = 'https://www.youtube.com' + JSON.parse(SubstringByCircumfix(
			watchHtml,
			'">(function() {window.ytplayer={};\nytcfg.set(',
			'); window.ytcfg.obfuscatedData_ = [];var setMessage=functi',
		)).PLAYER_JS_URL
		let sigFn
		let nFn
		if (playerJsCacheOrNull === null || playerJsCacheOrNull.uri !== playerJsUri) {
			playerJsResOrNull = await HttpsGetRes(playerJsUri)
			playerJsResBodyOrNull = await bufferHttpResBody(playerJsResOrNull)
			const playerJs = playerJsResBodyOrNull.toString()
			sigFn = (() => {
				const name = SubstringByCircumfix(playerJs, '(a,!0);a.set("alr","yes");c&&(c=', '(')
				const body = SubstringByCircumfix(playerJs, ';\n' + name + '=function(a){', '};')
				const helperName = SubstringByCircumfix(body, 'a=a.split("");', '.')
				const helperBody = SubstringByCircumfix(playerJs, ';var ' + helperName + '={', '};')
				return new Function('a', 'var ' + helperName + '={' + helperBody + '};' + body)
			})()
			nFn = (() => {
				const arrayName = SubstringByCircumfix(playerJs, '&&(b=a.get("n"))&&(b=', '[')
				const name = SubstringByCircumfix(playerJs, ';\nvar ' + arrayName + '=[', ']')
				return new Function('a', SubstringByCircumfix(
					playerJs,
					';\n' + name + '=function(a){',
					'return b.join("")};',
				) + 'return b.join("")')
			})()
			playerJsCacheOrNull = {nFn: nFn, sigFn: sigFn, uri: playerJsUri}
		} else {
			sigFn = playerJsCacheOrNull.sigFn
			nFn = playerJsCacheOrNull.nFn
		}
		const streamingData = JSON.parse(SubstringByCircumfix(
			watchHtml,
			'">var ytInitialPlayerResponse = ',
			';</script>',
		)).streamingData
		const formats = [
			...(streamingData.adaptiveFormats === undefined ? [] : streamingData.adaptiveFormats),
			...(streamingData.formats === undefined ? [] : streamingData.formats),
		]
		let audioFormatOrNull = null
		for (const itag of [251, 140, 18]) {
			const formatWithItagOrUndefined = formats.find((format) => {
				return format.itag === itag
			})
			if (formatWithItagOrUndefined !== undefined) {
				audioFormatOrNull = formatWithItagOrUndefined
				break
			}
		}
		let urlOrNull = null
		if (audioFormatOrNull.signatureCipher !== undefined) {
			const params = new URLSearchParams(audioFormatOrNull.signatureCipher)
			urlOrNull = new URL(params.get('url'))
			urlOrNull.searchParams.set('sig', sigFn(params.get('s')))
		} else if (audioFormatOrNull.url !== undefined) {
			urlOrNull = new URL(audioFormatOrNull.url)
		}
		if (urlOrNull.searchParams.has('n')) {
			urlOrNull.searchParams.set('n', nFn(urlOrNull.searchParams.get('n')))
		}
		uriOrNull = urlOrNull.toString()
	} catch (throwee) {
		threw = true
		throweeIfThrew = throwee
	}
	qsWriteDebuggingInfo('qsYtAudioUriOrNullInfo', {
		playerJsResBodyOrNull: playerJsResBodyOrNull === null ? null : playerJsResBodyOrNull.toString('base64'),
		playerJsResOrNull: playerJsResOrNull === null ? null : HttpResAsJsonValue(playerJsResOrNull),
		threw: threw,
		throweeIfThrew: threw ? ThroweeAsJsonValue(throweeIfThrew) : null,
		uriOrNull: uriOrNull,
		video: video,
		watchResBodyOrNull: watchResBodyOrNull === null ? null : watchResBodyOrNull.toString('base64'),
		watchResOrNull: watchResOrNull === null ? null : HttpResAsJsonValue(watchResOrNull),
	})
	return uriOrNull
}

export default QsYtAudioUriOrNull

==> queueshare-0.0.57.tgz: package/src/QsYtAudioUriOrNull.js <==
import SubstringByCircumfixOrNull from './SubstringByCircumfixOrNull.js'
import bufferHttpResBody from './bufferHttpResBody.js'
import HttpResAsJsonValue from './HttpResAsJsonValue.js'
import HttpsGetRes from './HttpsGetRes.js'
import qsWriteDebuggingInfo from './qsWriteDebuggingInfo.js'
import QueryString from './QueryString.js'
import SubstringByCircumfix from './SubstringByCircumfix.js'
import ThroweeAsJsonValue from './ThroweeAsJsonValue.js'
import Ungzipped from './Ungzipped.js'

let playerJsCacheOrNull = null
// todo use a uri->info map instead of caching just 1 uri? need to look at real-world debugging info to decide
const QsYtAudioUriOrNull = async (video) => {
	let watchResOrNull = null
	let watchResBodyOrNull = null
	let playerJsResOrNull = null
	let playerJsResBodyOrNull = null
	let uriOrNull = null
	let threw = false
	let throweeIfThrew
	try {
		watchResOrNull = await HttpsGetRes(
			'https://www.youtube.com/watch?' + QueryString({v: video}),
			{headers: {'Accept-Encoding': 'gzip'}},
		)
		watchResBodyOrNull = await bufferHttpResBody(watchResOrNull)
		const watchHtml = (await Ungzipped(watchResBodyOrNull)).toString()
		const playerJsUri = 'https://www.youtube.com' + JSON.parse(SubstringByCircumfix(
			watchHtml,
			'">(function() {window.ytplayer={};\nytcfg.set(',
			'); window.ytcfg.obfuscatedData_ = [];var setMessage=functi',
		)).PLAYER_JS_URL
		let sigFn
		let nFn
		if (playerJsCacheOrNull === null || playerJsCacheOrNull.uri !== playerJsUri) {
			playerJsResOrNull = await HttpsGetRes(playerJsUri)
			playerJsResBodyOrNull = await bufferHttpResBody(playerJsResOrNull)
			const playerJs = playerJsResBodyOrNull.toString()
			sigFn = (() => {
				const name = SubstringByCircumfix(playerJs, '(a,!0);a.set("alr","yes");c&&(c=', '(')
				const body = SubstringByCircumfix(playerJs, ';\n' + name + '=function(a){', '};')
				const helperName = SubstringByCircumfix(body, 'a=a.split("");', '.')
				const helperBody = SubstringByCircumfix(playerJs, ';var ' + helperName + '={', '};')
				return new Function('a', 'var ' + helperName + '={' + helperBody + '};' + body)
			})()
			nFn = (() => {
				const arrayName = SubstringByCircumfix(playerJs, '&&(b=a.get("n"))&&(b=', '[')
				const name = SubstringByCircumfix(playerJs, ';\nvar ' + arrayName + '=[', ']')
				return new Function('a', SubstringByCircumfix(
					playerJs,
					';\n' + name + '=function(a){',
					'return b.join("")};',
				) + 'return b.join("")')
			})()
			playerJsCacheOrNull = {nFn: nFn, sigFn: sigFn, uri: playerJsUri}
		}
		else {
			sigFn = playerJsCacheOrNull.sigFn
			nFn = playerJsCacheOrNull.nFn
		}
		// todo should probably figure out how to use regexes for this lol
		const streamingData = JSON.parse(SubstringByCircumfixOrNull(
			watchHtml,
			'">var ytInitialPlayerResponse = ',
			';var head = document.getElementsByTagName(\'head\')[0]; var meta = document.createElement(\'meta\'); meta.name = \'referrer\'; meta.content = \'origin\'; head.appendChild(meta); var noindexMeta = document.createElement(\'meta\'); noindexMeta.name = \'robots\'; noindexMeta.content = \'noindex\'; head.appendChild(noindexMeta);</script>',
		) || SubstringByCircumfixOrNull(
			watchHtml,
			'">var ytInitialPlayerResponse = ',
			';</script>',
		)).streamingData
		const formats = [
			...(streamingData.adaptiveFormats === undefined ? [] : streamingData.adaptiveFormats),
			...(streamingData.formats === undefined ? [] : streamingData.formats),
		]
		let audioFormatOrNull = null
		for (const itag of [251, 140, 18]) {
			const formatWithItagOrUndefined = formats.find((format) => {
				return format.itag === itag
			})
			if (formatWithItagOrUndefined !== undefined) {
				audioFormatOrNull = formatWithItagOrUndefined
				break
			}
		}
		let urlOrNull = null
		if (audioFormatOrNull.signatureCipher !== undefined) {
			const params = new URLSearchParams(audioFormatOrNull.signatureCipher)
			urlOrNull = new URL(params.get('url'))
			urlOrNull.searchParams.set('sig', sigFn(params.get('s')))
		}
		else if (audioFormatOrNull.url !== undefined) {
			urlOrNull = new URL(audioFormatOrNull.url)
		}
		if (urlOrNull.searchParams.has('n')) {
			urlOrNull.searchParams.set('n', nFn(urlOrNull.searchParams.get('n')))
		}
		uriOrNull = urlOrNull.toString()
	}
	catch (throwee) {
		threw = true
		throweeIfThrew = throwee
	}
	qsWriteDebuggingInfo('qsYtAudioUriOrNullInfo', {
		playerJsResBodyOrNull: playerJsResBodyOrNull === null ? null : playerJsResBodyOrNull.toString('base64'),
		playerJsResOrNull: playerJsResOrNull === null ? null : HttpResAsJsonValue(playerJsResOrNull),
		threw: threw,
		throweeIfThrew: threw ? ThroweeAsJsonValue(throweeIfThrew) : null,
		uriOrNull: uriOrNull,
		video: video,
		watchResBodyOrNull: watchResBodyOrNull === null ? null : watchResBodyOrNull.toString('base64'),
		watchResOrNull: watchResOrNull === null ? null : HttpResAsJsonValue(watchResOrNull),
	})
	return uriOrNull
}

export default QsYtAudioUriOrNull
